@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -103,7 +103,7 @@ gst_mms_class_init (GstMMSClass * klass)
   gst_element_class_add_pad_template (gstelement_class,
       gst_static_pad_template_get (&src_factory));
 
-  gst_element_class_set_metadata (gstelement_class,
+  gst_element_class_set_static_metadata (gstelement_class,
       "MMS streaming source", "Source/Network",
       "Receive data streamed via MSFT Multi Media Server protocol",
       "Maciej Katafiasz <mathrick@users.sourceforge.net>");
@@ -305,7 +305,7 @@ gst_mms_create (GstPushSrc * psrc, GstBuffer ** buf)
   guint8 *data;
   guint blocksize;
   gint result;
-  mms_off_t offset;
+  goffset offset;
 
   *buf = NULL;
 
@@ -339,7 +339,7 @@ gst_mms_create (GstPushSrc * psrc, GstBuffer ** buf)
   *buf = gst_buffer_new_wrapped (data, result);
   GST_BUFFER_OFFSET (*buf) = offset;
 
-  GST_LOG_OBJECT (mmssrc, "Returning buffer with offset %" G_GINT64_FORMAT
+  GST_LOG_OBJECT (mmssrc, "Returning buffer with offset %" G_GOFFSET_FORMAT
       " and size %u", offset, result);
 
   return GST_FLOW_OK;
@@ -609,8 +609,7 @@ gst_mms_uri_set_uri (GstURIHandler * handler, const gchar * uri,
   }
 
   GST_OBJECT_LOCK (src);
-  if (src->uri_name)
-    g_free (src->uri_name);
+  g_free (src->uri_name);
   src->uri_name = fixed_uri;
   GST_OBJECT_UNLOCK (src);
 
