@@ -34,7 +34,7 @@
  * <refsect2>
  * <title>Example pipeline</title>
  * |[
- * gst-launch -v filesrc location=1990s-nostalgia.xm ! modplug ! audioconvert ! alsasink
+ * gst-launch-1.0 -v filesrc location=1990s-nostalgia.xm ! modplug ! audioconvert ! alsasink
  * ]| Play a FastTracker xm file.
  * </refsect2>
  */
@@ -43,10 +43,6 @@
 #include "config.h"
 #endif
 
-/* FIXME 0.11: suppress warnings for deprecated API such as GStaticRecMutex
- * with newer GLib versions (>= 2.31.0) */
-#define GLIB_DISABLE_DEPRECATION_WARNINGS
-
 /* Required to not get an undefined warning
  * https://bugzilla.gnome.org/show_bug.cgi?id=613795
  */
@@ -95,7 +91,7 @@ enum
 #define DEFAULT_OVERSAMP         TRUE
 #define DEFAULT_NOISE_REDUCTION  TRUE
 
-#define FORMATS "{ "GST_AUDIO_NE (S32)", "GST_AUDIO_NE (S16)", U8 }"
+#define FORMATS "{ " GST_AUDIO_NE (S32) ", " GST_AUDIO_NE (S16) ", U8 }"
 
 static GstStaticPadTemplate modplug_src_template_factory =
 GST_STATIC_PAD_TEMPLATE ("src",
@@ -332,21 +328,12 @@ gst_modplug_do_seek (GstModPlug * modplug, GstEvent * event)
   gboolean flush;
   gint64 cur, stop;
   GstSegment seg;
-/* FIXME timestamp is set but not used */
-#if 0
-  guint64 timestamp;
-#endif
 
   if (modplug->frequency == 0)
     goto no_song;
 
-#if 0
-  timestamp = gst_util_uint64_scale_int (modplug->offset, GST_SECOND,
-      modplug->frequency);
-#endif
-
-  gst_event_parse_seek (event, &rate, &format, &flags,
-      &cur_type, &cur, &stop_type, &stop);
+  gst_event_parse_seek (event, &rate, &format, &flags, &cur_type, &cur,
+      &stop_type, &stop);
 
   if (format != GST_FORMAT_TIME)
     goto no_time;
@@ -357,8 +344,10 @@ gst_modplug_do_seek (GstModPlug * modplug, GstEvent * event)
 
   if (stop_type == GST_SEEK_TYPE_NONE)
     stop = GST_CLOCK_TIME_NONE;
+  if (!GST_CLOCK_TIME_IS_VALID (stop) && modplug->song_length > 0)
+    stop = modplug->song_length;
 
-  cur = CLAMP (cur, 0, modplug->song_length);
+  cur = CLAMP (cur, -1, modplug->song_length);
 
   GST_DEBUG_OBJECT (modplug, "seek to %" GST_TIME_FORMAT,
       GST_TIME_ARGS ((guint64) cur));
@@ -379,8 +368,6 @@ gst_modplug_do_seek (GstModPlug * modplug, GstEvent * event)
     gst_element_post_message (GST_ELEMENT (modplug),
         gst_message_new_segment_start (GST_OBJECT (modplug), format, cur));
   }
-  if (stop == -1 && modplug->song_length > 0)
-    stop = modplug->song_length;
 
   if (flush) {
     gst_pad_push_event (modplug->srcpad, gst_event_new_flush_stop (TRUE));
