@@ -49,40 +49,9 @@
 //#endif
 #endif
 
-#include <cstdlib>
 #include <errno.h>
-#include <math.h>
-#include <gst/gst.h>
-#include <arpa/inet.h>
 #include "MotionCells.h"
-
-uint64_t ntohl64 (uint64_t val);
-uint64_t htonl64 (uint64_t val);
-
-uint64_t
-ntohl64 (uint64_t val)
-{
-  uint64_t res64;
-  uint32_t low = (uint32_t) (val & 0x00000000FFFFFFFFLL);
-  uint32_t high = (uint32_t) ((val & 0xFFFFFFFF00000000LL) >> 32);
-  low = ntohl (low);
-  high = ntohl (high);
-  res64 = (uint64_t) high + (((uint64_t) low) << 32);
-  return res64;
-}
-
-
-uint64_t
-htonl64 (uint64_t val)
-{
-  uint64_t res64;
-  uint32_t low = (uint32_t) (val & 0x00000000FFFFFFFFLL);
-  uint32_t high = (uint32_t) ((val & 0xFFFFFFFF00000000LL) >> 32);
-  low = htonl (low);
-  high = htonl (high);
-  res64 = (uint64_t) high + (((uint64_t) low) << 32);
-  return res64;
-}
+#include <opencv2/imgproc/imgproc_c.h>
 
 MotionCells::MotionCells ()
 {
@@ -104,7 +73,12 @@ MotionCells::MotionCells ()
   m_beta = 0.5;
   m_useAlpha = false;
   m_isVisible = false;
-
+  m_pCells = NULL;
+  m_gridx = 0;
+  m_gridy = 0;
+  m_cellwidth = 0;
+  m_cellheight = 0;
+  m_sensitivity = 0;
 }
 
 MotionCells::~MotionCells ()
@@ -146,7 +120,8 @@ MotionCells::performDetectionMotionCells (IplImage * p_frame,
       : p_framerate <= 10 ? sumframecnt = 2
       : p_framerate <= 15 ? sumframecnt = 3
       : p_framerate <= 20 ? sumframecnt = 4
-      : p_framerate <= 25 ? sumframecnt = 5 : sumframecnt = 0 : sumframecnt = 0;
+      : p_framerate <= 25 ? sumframecnt = 5
+      : p_framerate <= 30 ? sumframecnt = 6 : sumframecnt = 0 : sumframecnt = 0;
 
   m_framecnt++;
   m_changed_datafile = p_changed_datafile;
@@ -199,7 +174,6 @@ MotionCells::performDetectionMotionCells (IplImage * p_frame,
     if (motionmaskcells_count > 0)
       performMotionMask (motionmaskcellsidx, motionmaskcells_count);
     if (getIsNonZero (m_pbwImage)) {    //detect Motion
-      GST_DEBUG ("DETECT MOTION \n");
       if (m_MotionCells.size () > 0)    //it contains previous motioncells what we used when frames dropped
         m_MotionCells.clear ();
       if (transparencyimg)
@@ -359,20 +333,20 @@ MotionCells::initDataFile (char *p_datafile, gint64 starttime)  //p_date is incr
     }
   } else
     mc_savefile = NULL;
-  bzero (&m_header, sizeof (MotionCellHeader));
-  m_header.headersize = htonl (MC_HEADER);
-  m_header.type = htonl (MC_TYPE);
-  m_header.version = htonl (MC_VERSION);
+  memset (&m_header, 0, sizeof (MotionCellHeader));
+  m_header.headersize = GINT32_TO_BE (MC_HEADER);
+  m_header.type = GINT32_TO_BE (MC_TYPE);
+  m_header.version = GINT32_TO_BE (MC_VERSION);
   //it needs these bytes
   m_header.itemsize =
-      htonl ((int) ceil (ceil (m_gridx * m_gridy / 8.0) / 4.0) * 4 +
+      GINT32_TO_BE ((int) ceil (ceil (m_gridx * m_gridy / 8.0) / 4.0) * 4 +
       sizeof (mcd.timestamp));
-  m_header.gridx = htonl (m_gridx);
-  m_header.gridy = htonl (m_gridy);
-  m_header.starttime = htonl64 (starttime);
+  m_header.gridx = GINT32_TO_BE (m_gridx);
+  m_header.gridy = GINT32_TO_BE (m_gridy);
+  m_header.starttime = GINT64_TO_BE (starttime);
 
   snprintf (m_header.name, sizeof (m_header.name), "%s %dx%d", MC_VERSIONTEXT,
-      ntohl (m_header.gridx), ntohl (m_header.gridy));
+      GINT32_FROM_BE (m_header.gridx), GINT32_FROM_BE (m_header.gridy));
   m_changed_datafile = false;
   return 0;
 }
@@ -382,7 +356,7 @@ MotionCells::saveMotionCells (gint64 timestamp_millisec)
 {
 
   MotionCellData mc_data;
-  mc_data.timestamp = htonl (timestamp_millisec);
+  mc_data.timestamp = GINT32_TO_BE (timestamp_millisec);
   mc_data.data = NULL;
   //There is no datafile
   if (mc_savefile == NULL)
@@ -402,7 +376,7 @@ MotionCells::saveMotionCells (gint64 timestamp_millisec)
 
   mc_data.data =
       (char *) calloc (1,
-      ntohl (m_header.itemsize) - sizeof (mc_data.timestamp));
+      GINT32_FROM_BE (m_header.itemsize) - sizeof (mc_data.timestamp));
   if (mc_data.data == NULL) {
     //fprintf(stderr, "%s %d:saveMotionCells:calloc:%d (%s)\n", __FILE__, __LINE__, errno,
     //strerror(errno));
@@ -413,7 +387,7 @@ MotionCells::saveMotionCells (gint64 timestamp_millisec)
 
   for (unsigned int i = 0; i < m_MotionCells.size (); i++) {
     int bitnum =
-        m_MotionCells.at (i).lineidx * ntohl (m_header.gridx) +
+        m_MotionCells.at (i).lineidx * GINT32_FROM_BE (m_header.gridx) +
         m_MotionCells.at (i).colidx;
     int bytenum = (int) floor (bitnum / 8.0);
     int shift = bitnum - bytenum * 8;
@@ -432,7 +406,7 @@ MotionCells::saveMotionCells (gint64 timestamp_millisec)
   }
 
   if (fwrite (mc_data.data,
-          ntohl (m_header.itemsize) - sizeof (mc_data.timestamp), 1,
+          GINT32_FROM_BE (m_header.itemsize) - sizeof (mc_data.timestamp), 1,
           mc_savefile) != 1) {
     //fprintf(stderr, "%s %d:saveMotionCells:fwrite:%d (%s)\n", __FILE__, __LINE__, errno,
     //strerror(errno));
