@@ -70,6 +70,11 @@ struct _GstCvSobel
   gint x_order;
   gint y_order;
   gint aperture_size;
+  gboolean mask;
+
+  IplImage *cvCSobel;
+  IplImage *cvGray;
+  IplImage *cvSobel;
 };
 
 struct _GstCvSobelClass
