@@ -29,18 +29,14 @@
 #include <gst/gst.h>
 #include <gst/video/video.h>
 
-#include <cv.h>
 
-gboolean
-gst_opencv_get_ipldepth (gint depth, gint bpp, gint * ipldepth);
+G_BEGIN_DECLS
 
 gboolean gst_opencv_parse_iplimage_params_from_caps
     (GstCaps * caps, gint * width, gint * height, gint * depth,
     gint * channels, GError ** err);
-gboolean gst_opencv_parse_iplimage_params_from_structure
-    (GstStructure * structure, gint * width, gint * height, gint * depth,
-    gint * channels, GError ** err);
 
 GstCaps * gst_opencv_caps_from_cv_image_type (int cv_type);
+G_END_DECLS
 
 #endif /* __GST_OPENCV_UTILS__ */
