@@ -1039,7 +1039,11 @@ gst_openjpeg_dec_handle_frame (GstVideoDecoder * decoder,
   opj_stream_set_write_function (stream, write_fn);
   opj_stream_set_skip_function (stream, skip_fn);
   opj_stream_set_seek_function (stream, seek_fn);
+#ifdef HAVE_OPENJPEG_2_1
+  opj_stream_set_user_data (stream, &mstream, NULL);
+#else
   opj_stream_set_user_data (stream, &mstream);
+#endif
   opj_stream_set_user_data_length (stream, mstream.size);
 
   image = NULL;
