@@ -856,6 +856,9 @@ gst_openjpeg_enc_handle_frame (GstVideoEncoder * encoder,
     goto fill_image_error;
   gst_video_frame_unmap (&vframe);
 
+  if (vframe.info.finfo->flags & GST_VIDEO_FORMAT_FLAG_RGB) {
+    self->params.tcp_mct = 1;
+  }
   opj_setup_encoder (enc, &self->params, image);
 
 #ifdef HAVE_OPENJPEG_1
@@ -901,7 +904,11 @@ gst_openjpeg_enc_handle_frame (GstVideoEncoder * encoder,
   opj_stream_set_write_function (stream, write_fn);
   opj_stream_set_skip_function (stream, skip_fn);
   opj_stream_set_seek_function (stream, seek_fn);
+#ifdef HAVE_OPENJPEG_2_1
+  opj_stream_set_user_data (stream, &mstream, NULL);
+#else
   opj_stream_set_user_data (stream, &mstream);
+#endif
   opj_stream_set_user_data_length (stream, mstream.size);
 
   if (!opj_start_compress (enc, image, stream))
@@ -936,6 +943,7 @@ gst_openjpeg_enc_handle_frame (GstVideoEncoder * encoder,
           mstream.size, NULL, (GDestroyNotify) g_free));
 #endif
 
+  GST_VIDEO_CODEC_FRAME_SET_SYNC_POINT (frame);
   ret = gst_video_encoder_finish_frame (encoder, frame);
 
   return ret;
