@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /* let's not forget to mention that all this was based on aasink ;-) */
@@ -944,7 +944,9 @@ gst_sdlvideosink_navigation_send_event (GstNavigation * navigation,
 {
   GstSDLVideoSink *sdlvideosink = GST_SDLVIDEOSINK (navigation);
   GstEvent *event;
-  GstVideoRectangle src, dst, result;
+  GstVideoRectangle dst = { 0, };
+  GstVideoRectangle src = { 0, };
+  GstVideoRectangle result;
   double x, y, old_x, old_y;
   GstPad *pad = NULL;
 
