@@ -437,8 +437,11 @@ _get_gl_context_for_thread_unlocked (GstGLDisplay * display, GThread * thread)
       continue;
     }
 
+    if (thread == NULL)
+      return context;
+
     context_thread = gst_gl_context_get_thread (context);
-    if (thread != NULL && thread == context_thread) {
+    if (thread != context_thread) {
       g_thread_unref (context_thread);
       gst_object_unref (context);
       prev = l;
@@ -497,7 +500,7 @@ _check_collision (GstGLContext * context, GstGLContext * collision)
     goto out;
   }
 
-  if (collision == context) {
+  if (thread == collision_thread) {
     ret = TRUE;
     goto out;
   }
@@ -543,6 +546,13 @@ gst_gl_display_add_context (GstGLDisplay * display, GstGLContext * context)
   if (thread) {
     collision = _get_gl_context_for_thread_unlocked (display, thread);
     g_thread_unref (thread);
+
+    /* adding the same context is a no-op */
+    if (context == collision) {
+      ret = TRUE;
+      goto out;
+    }
+
     if (_check_collision (context, collision)) {
       ret = FALSE;
       goto out;
