@@ -28,7 +28,7 @@
  * <refsect2>
  * <title>Example launch line</title>
  * |[
- * gst-launch audiotestsrc ! audioconvert ! wavescope ! ximagesink
+ * gst-launch-1.0 audiotestsrc ! audioconvert ! wavescope ! ximagesink
  * ]|
  * </refsect2>
  */
@@ -176,8 +176,7 @@ gst_wave_scope_setup (GstAudioVisualizer * bscope)
 {
   GstWaveScope *scope = GST_WAVE_SCOPE (bscope);
 
-  if (scope->flt)
-    g_free (scope->flt);
+  g_free (scope->flt);
 
   scope->flt = g_new0 (gdouble, 6 * GST_AUDIO_INFO_CHANNELS (&bscope->ainfo));
 
