@@ -47,6 +47,20 @@
  * his code.
  */
 
+/**
+ * SECTION:element-diffuse
+ * @see_also: geometrictransform
+ *
+ * Diffuse is a geometric image transform element. It diffuses the image by
+ * moving its pixels in random directions.
+ *
+ * <refsect2>
+ * <title>Example launch line</title>
+ * |[
+ * gst-launch-1.0 -v videotestsrc ! diffuse ! videoconvert ! autovideosink
+ * ]|
+ * </refsect2>
+ */
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -182,17 +196,15 @@ gst_diffuse_class_init (GstDiffuseClass * klass)
   gstelement_class = (GstElementClass *) klass;
   gstgt_class = (GstGeometricTransformClass *) klass;
 
-  parent_class = g_type_class_peek_parent (klass);
-
   gst_element_class_set_static_metadata (gstelement_class,
       "diffuse",
       "Transform/Effect/Video",
       "Diffuses the image by moving its pixels in random directions",
       "Thiago Santos<thiago.sousa.santos@collabora.co.uk>");
 
-  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_diffuse_finalize);
-  gobject_class->set_property = GST_DEBUG_FUNCPTR (gst_diffuse_set_property);
-  gobject_class->get_property = GST_DEBUG_FUNCPTR (gst_diffuse_get_property);
+  gobject_class->finalize = gst_diffuse_finalize;
+  gobject_class->set_property = gst_diffuse_set_property;
+  gobject_class->get_property = gst_diffuse_get_property;
 
   g_object_class_install_property (gobject_class, PROP_SCALE,
       g_param_spec_double ("scale", "scale",
