@@ -47,6 +47,19 @@
  * his code.
  */
 
+/**
+ * SECTION:element-sphere
+ * @see_also: geometrictransform
+ *
+ * The sphere element applies a 'sphere' geometric transform to the image.
+ *
+ * <refsect2>
+ * <title>Example launch line</title>
+ * |[
+ * gst-launch-1.0 -v videotestsrc ! sphere ! videoconvert ! autovideosink
+ * ]|
+ * </refsect2>
+ */
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -116,13 +129,6 @@ gst_sphere_get_property (GObject * object, guint prop_id,
   }
 }
 
-/* Clean up */
-static void
-gst_sphere_finalize (GObject * obj)
-{
-  G_OBJECT_CLASS (parent_class)->finalize (obj);
-}
-
 /* TODO we could have horizontal and vertical 'radius' */
 static gboolean
 sphere_map (GstGeometricTransform * gt, gint x, gint y, gdouble * in_x,
@@ -187,17 +193,14 @@ gst_sphere_class_init (GstSphereClass * klass)
   gstelement_class = (GstElementClass *) klass;
   gstgt_class = (GstGeometricTransformClass *) klass;
 
-  parent_class = g_type_class_peek_parent (klass);
-
   gst_element_class_set_static_metadata (gstelement_class,
       "sphere",
       "Transform/Effect/Video",
       "Applies 'sphere' geometric transform to the image",
       "Thiago Santos<thiago.sousa.santos@collabora.co.uk>");
 
-  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_sphere_finalize);
-  gobject_class->set_property = GST_DEBUG_FUNCPTR (gst_sphere_set_property);
-  gobject_class->get_property = GST_DEBUG_FUNCPTR (gst_sphere_get_property);
+  gobject_class->set_property = gst_sphere_set_property;
+  gobject_class->get_property = gst_sphere_get_property;
 
   g_object_class_install_property (gobject_class, PROP_REFRACTION,
       g_param_spec_double ("refraction", "refraction",
