@@ -1391,7 +1391,7 @@ get_compatible_profile_caps (GstH264SPS * sps)
     case GST_H264_PROFILE_BASELINE:
       if (sps->constraint_set1_flag) {  /* A.2.1 */
         static const gchar *profile_array[] =
-            { "constrained-baseline", "main", "high", "high-10", "high-4:2:2",
+            { "baseline", "main", "high", "high-10", "high-4:2:2",
           "high-4:4:4", NULL
         };
         profiles = profile_array;
@@ -1625,7 +1625,9 @@ get_profile_string (GstH264SPS * sps)
 static const gchar *
 get_level_string (GstH264SPS * sps)
 {
-  if ((sps->level_idc == 11 && sps->constraint_set3_flag)
+  if (sps->level_idc == 0)
+    return NULL;
+  else if ((sps->level_idc == 11 && sps->constraint_set3_flag)
       || sps->level_idc == 9)
     return "1b";
   else if (sps->level_idc % 10 == 0)
