@@ -409,14 +409,14 @@ caps_from_amc_format (GstAmcFormat * amc_format)
 unsupported_profile:
   GST_ERROR ("Unsupport amc profile id %d", amc_profile);
   g_free (mime);
-  gst_object_unref (caps);
+  gst_caps_unref (caps);
 
   return NULL;
 
 unsupported_level:
   GST_ERROR ("Unsupport amc level id %d", amc_level);
   g_free (mime);
-  gst_object_unref (caps);
+  gst_caps_unref (caps);
 
   return NULL;
 }
@@ -1337,7 +1337,7 @@ gst_amc_video_enc_set_format (GstVideoEncoder * encoder,
 
 quit:
   if (allowed_caps)
-    gst_object_unref (allowed_caps);
+    gst_caps_unref (allowed_caps);
 
   if (format)
     gst_amc_format_free (format);
@@ -1469,8 +1469,10 @@ again:
   /* Copy the buffer content in chunks of size as requested
    * by the port */
   buf = gst_amc_codec_get_input_buffer (self->codec, idx, &err);
-  if (!buf)
+  if (err)
     goto failed_to_get_input_buffer;
+  else if (!buf)
+    goto got_null_input_buffer;
 
   memset (&buffer_info, 0, sizeof (buffer_info));
   buffer_info.offset = 0;
@@ -1539,6 +1541,13 @@ failed_to_get_input_buffer:
     gst_video_codec_frame_unref (frame);
     return GST_FLOW_ERROR;
   }
+got_null_input_buffer:
+  {
+    GST_ELEMENT_ERROR (self, LIBRARY, SETTINGS, (NULL),
+        ("Got no input buffer"));
+    gst_video_codec_frame_unref (frame);
+    return GST_FLOW_ERROR;
+  }
 buffer_fill_error:
   {
     GST_ELEMENT_ERROR (self, RESOURCE, WRITE, (NULL),
