@@ -90,6 +90,7 @@ struct _ShmArea
   int id;
 
   int use_count;
+  int is_writer;
 
   int shm_fd;
 
@@ -286,6 +287,8 @@ sp_open_shm (char *path, int id, mode_t perms, size_t size)
 
   area->shm_area_len = size;
 
+  area->is_writer = (path == NULL);
+
 
   if (path)
     flags = O_RDONLY;
@@ -320,6 +323,7 @@ sp_open_shm (char *path, int id, mode_t perms, size_t size)
 
     prot = PROT_READ | PROT_WRITE;
   } else {
+    area->shm_area_name = strdup (path);
     prot = PROT_READ;
   }
 
@@ -353,7 +357,8 @@ sp_close_shm (ShmArea * area)
     close (area->shm_fd);
 
   if (area->shm_area_name) {
-    shm_unlink (area->shm_area_name);
+    if (area->is_writer)
+      shm_unlink (area->shm_area_name);
     free (area->shm_area_name);
   }
 
@@ -428,8 +433,10 @@ void
 sp_writer_close (ShmPipe * self, sp_buffer_free_callback callback,
     void *user_data)
 {
-  if (self->main_socket >= 0)
+  if (self->main_socket >= 0) {
+    shutdown (self->main_socket, SHUT_RDWR);
     close (self->main_socket);
+  }
 
   if (self->socket_path) {
     unlink (self->socket_path);
@@ -662,13 +669,15 @@ sp_client_recv (ShmPipe * self, char **buf)
       assert (cb.payload.new_shm_area.path_size > 0);
       assert (cb.payload.new_shm_area.size > 0);
 
-      area_name = malloc (cb.payload.new_shm_area.path_size);
+      area_name = malloc (cb.payload.new_shm_area.path_size + 1);
       retval = recv (self->main_socket, area_name,
           cb.payload.new_shm_area.path_size, 0);
       if (retval != cb.payload.new_shm_area.path_size) {
         free (area_name);
         return -3;
       }
+      /* Ensure area_name is NULL terminated */
+      area_name[retval] = 0;
 
       newarea = sp_open_shm (area_name, cb.area_id, 0,
           cb.payload.new_shm_area.size);
@@ -723,15 +732,11 @@ sp_writer_recv (ShmPipe * self, ShmClient * client, void **tag)
         if (buf->shm_area->id == cb.area_id &&
             buf->offset == cb.payload.ack_buffer.offset) {
           return sp_shmbuf_dec (self, buf, prev_buf, client, tag);
-          break;
         }
         prev_buf = buf;
       }
 
-      if (!buf)
-        return -2;
-
-      break;
+      return -2;
     default:
       return -99;
   }
@@ -840,6 +845,7 @@ sp_writer_accept_client (ShmPipe * self)
   return client;
 
 error:
+  shutdown (fd, SHUT_RDWR);
   close (fd);
   return NULL;
 }
@@ -892,6 +898,7 @@ sp_writer_close_client (ShmPipe * self, ShmClient * client,
   ShmBuffer *buffer = NULL, *prev_buf = NULL;
   ShmClient *item = NULL, *prev_item = NULL;
 
+  shutdown (client->fd, SHUT_RDWR);
   close (client->fd);
 
 again:
@@ -935,6 +942,15 @@ sp_get_fd (ShmPipe * self)
   return self->main_socket;
 }
 
+const gchar *
+sp_get_shm_area_name (ShmPipe * self)
+{
+  if (self->shm_area)
+    return self->shm_area->shm_area_name;
+
+  return NULL;
+}
+
 int
 sp_writer_get_client_fd (ShmClient * client)
 {
