@@ -19,6 +19,12 @@
 
 #include "kshelpers.h"
 
+/* This plugin is from the era of Windows XP and uses APIs that have been
+ * deprecated since then. Let's pretend we're Windows XP too so that Windows
+ * lets us use that deprecated API. */
+#define NTDDI_VERSION NTDDI_WINXP
+#define _WIN32_WINNT _WIN32_WINNT_WINXP
+
 #include <ksmedia.h>
 #include <setupapi.h>
 #include <gst/gst.h>
